-------------------------------------------------------------------------------
-- AFK & DND v0.5a
-------------------------------------------------------------------------------
--
-- Copyright (c) 2015 Ales Machat (Garkin)
--
-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.
--
-------------------------------------------------------------------------------
--
-- DISCLAIMER:
--
-- This Add-on is not created by, affiliated with or sponsored by ZeniMax
-- Media Inc. or its affiliates. The Elder Scrolls® and related logos are
-- registered trademarks or trademarks of ZeniMax Media Inc. in the United
-- States and/or other countries. All rights reserved.
--
-- You can read the full terms at:
-- https://account.elderscrollsonline.com/add-on-terms
--
-------------------------------------------------------------------------------

local ValidateChatChannel_Orig
local savedVariables

local function TogglePlayerStatus(status)
   local newStatus = GetPlayerStatus() == status and PLAYER_STATUS_ONLINE or status  
   SelectPlayerStatus(newStatus)
end

local function WakeUpWhenMoving()
    if IsPlayerMoving() then
        EVENT_MANAGER:UnregisterForUpdate("__AFK__")
        TogglePlayerStatus(PLAYER_STATUS_AWAY)
    end
end

local function OnStatusChanged(eventCode, oldStatus, newStatus)
    d(zo_strformat("|t28:28:<<X:1>>|t <<2>>", GetPlayerStatusIcon(newStatus), GetString("SI_PLAYERSTATUS", newStatus)))

    savedVariables["status"] = newStatus

--     if newStatus == PLAYER_STATUS_AWAY then
--         EVENT_MANAGER:RegisterForUpdate("__AFK__", 500, WakeUpWhenMoving)
--     end
end

local function ValidateChatChannel_Hook(self)
    local valid = ValidateChatChannel_Orig(self)
    if valid == true then
        local text = self.textEntry:GetText()
        if #text > 0 then
            if self.currentChannel == CHAT_CHANNEL_WHISPER_SENT then
                if IsFriend(self.currentTarget) then
                    for i = 1, GetNumFriends() do
                        local displayName, _, playerStatus = GetFriendInfo(i)
                        local hasCharacter, characterName = GetFriendCharacterInfo(i)
                        if self.currentTarget == displayName or (hasCharacter and self.currentTarget == characterName) then
                            if playerStatus ~= PLAYER_STATUS_ONLINE then
                                d(zo_strformat("<<C:1>> is |t28:28:<<X:2>>|t <<3>>", ZO_LinkHandler_CreatePlayerLink(self.currentTarget), GetPlayerStatusIcon(playerStatus), GetString("SI_PLAYERSTATUS", playerStatus)))
                                return true
                            end
                        end
                    end
                else
                    for i = 1, GetNumGuilds() do
                        for j = 1, GetNumGuildMembers(i) do
                            local displayName, _, _, playerStatus = GetGuildMemberInfo(i, j)
                            local hasCharacter, characterName = GetGuildMemberCharacterInfo(i, j)
                            if self.currentTarget == displayName or (hasCharacter and self.currentTarget == characterName) then
                                if playerStatus ~= PLAYER_STATUS_ONLINE then
                                    d(zo_strformat("<<C:1>> is |t28:28:<<X:2>>|t <<3>>", ZO_LinkHandler_CreatePlayerLink(self.currentTarget), GetPlayerStatusIcon(playerStatus), GetString("SI_PLAYERSTATUS", playerStatus)))
                                    return true
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    return valid
end

local function OnAddOnLoaded(eventCode, addonName)
    if addonName == "AFK" then
        EVENT_MANAGER:UnregisterForEvent("__AFK__", eventCode)

        savedVariables = ZO_SavedVars:New("AFK_SavedVariables", 1, "socialstatus", { ["status"] = PLAYER_STATUS_ONLINE })

        SelectPlayerStatus(savedVariables["status"])

        EVENT_MANAGER:RegisterForEvent("__AFK__", EVENT_PLAYER_STATUS_CHANGED, OnStatusChanged)

        ValidateChatChannel_Orig = CHAT_SYSTEM.ValidateChatChannel
        CHAT_SYSTEM.ValidateChatChannel = ValidateChatChannel_Hook

        SLASH_COMMANDS["/dnd"] = function() TogglePlayerStatus(PLAYER_STATUS_DO_NOT_DISTURB) end 
        SLASH_COMMANDS["/afk"] = function() TogglePlayerStatus(PLAYER_STATUS_AWAY) end
        SLASH_COMMANDS["/on"]  = function() TogglePlayerStatus(PLAYER_STATUS_ONLINE) end
        SLASH_COMMANDS["/off"] = function() TogglePlayerStatus(PLAYER_STATUS_OFFLINE) end
    end
end

EVENT_MANAGER:RegisterForEvent("__AFK__", EVENT_ADD_ON_LOADED, OnAddOnLoaded)
